#include <Adafruit_INA219.h>

static Adafruit_INA219 ina219;

static const int ledPin{13};

static const char banner[]{
  "  time  |  voltage  |  current  |  power\r\n"
  "--------+-----------+-----------+---------\r\n"
};
static char buffer[50];

static bool paused = true;
static unsigned displayOffset;
static unsigned lastUpdate;
//static float mBuffer[100];
static const unsigned updateDelay = 50;

void setup() {
  bool success;

  Serial.begin(1500000);

  pinMode(ledPin, OUTPUT);
  do {
    success = ina219.begin();
    if (success) {
      ina219.setCalibration_16V_400mA();
      success = ina219.success();
    }
    if (success) {
      ina219.powerSave(paused);
      success = ina219.success();
    }

    digitalWrite(ledPin, HIGH);
    delay(50);
    digitalWrite(ledPin, LOW);
    delay(50);
  } while (!success);
  pinMode(ledPin, INPUT);
}

void loop() {
  unsigned now{millis()};

  if (paused)
    return;
  if (!Serial.availableForWrite())
    return;
  if (now < lastUpdate + updateDelay)
    return;
  lastUpdate = now;

  now -= displayOffset;
#if 0
  unsigned seconds  = now / 1000;
  unsigned fraction = now % 1000;
  if (seconds % 100 == 0 && (!lastUpdate || fraction < updateDelay))
    Serial.print(banner);

  float voltage = ina219.getBusVoltage_V();
  float current = ina219.getCurrent_mA();
  float power   = ina219.getPower_mW();
  snprintf(buffer, sizeof(buffer), " %6u | %7.3f V | %6.1f mA | %4.0f mW\r\n",
           seconds, voltage, current, power);
#else
  float voltage = ina219.getBusVoltage_V();
  float current = ina219.getCurrent_mA();
  //snprintf(buffer, sizeof(buffer), "%d %.1f\r\n", now, voltage * current);
  snprintf(buffer, sizeof(buffer), "%.1f\r\n", voltage * current);
#endif
  Serial.print(buffer);
}

void serialEvent() {
  while (Serial.available()) {
    if (Serial.read() == '\r') {
      displayOffset = millis();
      paused = !paused;
      ina219.powerSave(paused);
      delay(1);
    }
  }
}
